#
# Scripts Related Tests
#
from AccessControl import Unauthorized

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase

from Products.LinguaPlone.tests import LinguaPloneTestCase
from Products.LinguaPlone.tests.utils import makeContent
from Products.LinguaPlone.tests.utils import makeTranslation


class TestLanguageLists(LinguaPloneTestCase.LinguaPloneTestCase):

    def afterSetUp(self):
        self.addLanguage('de')
        self.addLanguage('fr')
        self.addLanguage('es')
        self.setLanguage('en')
        # Speed things up
        self.portal._delObject('portal_catalog')
        self.english = makeContent(self.folder, 'SimpleType', 'doc')
        self.english.setLanguage('en')

    def testGetTranslatedLanguagesDefault(self):
        # List should contain only the canonical translation
        langs = self.english.getTranslatedLanguages()
        langs = [l[0] for l in langs]
        self.assertEqual(langs, ['en'])

    def testGetTranslatedLanguages(self):
        # List should contain all existing translations
        makeTranslation(self.english, 'de')
        makeTranslation(self.english, 'fr')
        langs = self.english.getTranslatedLanguages()
        langs = [l[0] for l in langs]
        self.assertEqual(langs, ['de', 'en', 'fr'])

    def testGetTranslatedLanguagesWithNoLanguage(self):
        # List should be emtpy, no language defined
        # Fake a translatable object without a language defined
        self.english.setLanguage('')
        langs = self.english.getTranslatedLanguages()
        self.assertEqual(langs, [])

    def testGetTranslatedLanguagesWithUnusedLanguage(self):
        # List should be emtpy, pt is not selected at portal_languages
        self.english.setLanguage('pt')
        langs = self.english.getTranslatedLanguages()
        self.assertEqual(langs, [])

    def testGetUntranslatedLanguagesEmpty(self):
        # List should be emtpy
        makeTranslation(self.english, 'de')
        makeTranslation(self.english, 'fr')
        makeTranslation(self.english, 'es')
        langs = self.english.getUntranslatedLanguages()
        self.assertEqual(langs, [])

    def testGetUntranslatedLanguages(self):
        # List should contain all translations waiting to be done
        makeTranslation(self.english, 'de')
        langs = self.english.getUntranslatedLanguages()
        langs = [l[0] for l in langs]
        self.assertEqual(langs, ['es', 'fr'])

    def testGetDeletableLanguagesEmpty(self):
        # List should be emtpy
        langs = self.english.getDeletableLanguages()
        self.assertEqual(langs, [])

    def testGetDeletableLanguages(self):
        # List should contain all translations except the canonical one
        makeTranslation(self.english, 'de')
        makeTranslation(self.english, 'fr')
        langs = self.english.getDeletableLanguages()
        langs = [l[0] for l in langs]
        self.assertEqual(langs, ['de', 'fr'])

def getSelectorValues(selectorData, key):
    result = {}
    for dict in selectorData:
        result[dict['Language']] = dict.get(key, None)
    return result

class TestSelectorData(LinguaPloneTestCase.LinguaPloneTestCase):

    def afterSetUp(self):
        self.addLanguage('de')
        self.addLanguage('fr')
        # Speed things up
        self.portal._delObject('portal_catalog')
        self.english = makeContent(self.folder, 'SimpleType', 'doc')
        self.english.setLanguage('en')

    def testSelectorData(self):
        self.french = makeTranslation(self.english, 'fr')
        sd = self.english.languageSelectorData()
        change_urls = getSelectorValues(sd, 'change_url')
        target_url = "%s/view?set_language=%s" % (self.english.absolute_url(), 'en')
        self.assertEqual(change_urls['en'], target_url)
        target_url = "%s/view?set_language=%s" % (self.french.absolute_url(), 'fr')
        self.assertEqual(change_urls['fr'], target_url)
        target_url = "%s/not_available_lang/view?set_language=%s" % (self.english.absolute_url(), 'de')
        self.assertEqual(change_urls['de'], target_url)
        availables = getSelectorValues(sd, 'available')
        self.assertEqual(availables['en'], True)
        self.assertEqual(availables['fr'], True)
        self.assertEqual(availables['de'], False)
        currents = getSelectorValues(sd, 'current')
        self.assertEqual(currents['en'], True)
        self.assertEqual(currents['fr'], False)
        self.assertEqual(currents['de'], False)
        alts = getSelectorValues(sd, 'alt')
        self.failUnless('Switch language' in alts['en'])
        self.failIf('not available' in alts['en'])
        self.failUnless('Switch language' in alts['fr'])
        self.failIf('not available' in alts['fr'])
        self.failUnless('Switch language' in alts['de'])
        self.failUnless('not available' in alts['de'])
    
    def testUnauthorizedTranslation(self):
        # create second user in order to hide content
        uf = self.portal.acl_users
        uf.userFolderAddUser('user2', 'user2', ['Member'], [])
        self.folder.manage_setLocalRoles('user2', ['Owner'])
        self.login('user2')
        # make a french translation
        self.french = makeTranslation(self.english, 'fr')
        # hide translation to make it unauthorized to default user
        self.french.folder_localrole_set(use_acquisition=False)
        self.portal.portal_workflow.doActionFor(self.french, 'hide')
        self.login()
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse, self.french.getId()+'/Title')
        sd = self.english.languageSelectorData()
        change_urls = getSelectorValues(sd, 'change_url')
        target_url = "%s/view?set_language=%s" % (self.english.absolute_url(), 'en')
        self.assertEqual(change_urls['en'], target_url)
        target_url = "%s/not_available_lang/view?set_language=%s" % (self.english.absolute_url(), 'fr')
        self.assertEqual(change_urls['fr'], target_url)
        target_url = "%s/not_available_lang/view?set_language=%s" % (self.english.absolute_url(), 'de')
        self.assertEqual(change_urls['de'], target_url)
        availables = getSelectorValues(sd, 'available')
        self.assertEqual(availables['en'], True)
        self.assertEqual(availables['fr'], False)
        self.assertEqual(availables['de'], False)
        currents = getSelectorValues(sd, 'current')
        self.assertEqual(currents['en'], True)
        self.assertEqual(currents['fr'], False)
        self.assertEqual(currents['de'], False)
        alts = getSelectorValues(sd, 'alt')
        self.failUnless('Switch language' in alts['en'])
        self.failIf('not available' in alts['en'])
        self.failUnless('Switch language' in alts['fr'])
        self.failUnless('not available' in alts['fr'])
        self.failUnless('Switch language' in alts['de'])
        self.failUnless('not available' in alts['de'])

    def testLanguageNeutral(self):
        self.neutral = makeContent(self.folder, 'SimpleType', 'neutral')
        sd = self.neutral.languageSelectorData()
        change_urls = getSelectorValues(sd, 'change_url')
        target_url = "%s/view?set_language=%s" % (self.neutral.absolute_url(), 'en')
        self.assertEqual(change_urls['en'], target_url)
        target_url = "%s/view?set_language=%s" % (self.neutral.absolute_url(), 'fr')
        self.assertEqual(change_urls['fr'], target_url)
        target_url = "%s/view?set_language=%s" % (self.neutral.absolute_url(), 'de')
        self.assertEqual(change_urls['de'], target_url)
        availables = getSelectorValues(sd, 'available')
        self.assertEqual(availables['en'], False)
        self.assertEqual(availables['fr'], False)
        self.assertEqual(availables['de'], False)
        currents = getSelectorValues(sd, 'current')
        self.assertEqual(currents['en'], True)
        self.assertEqual(currents['fr'], False)
        self.assertEqual(currents['de'], False)
        alts = getSelectorValues(sd, 'alt')
        self.failUnless('Switch language' in alts['en'])
        self.failUnless('language neutral' in alts['en'])
        self.failUnless('Switch language' in alts['fr'])
        self.failUnless('language neutral' in alts['fr'])
        self.failUnless('Switch language' in alts['de'])
        self.failUnless('language neutral' in alts['de'])

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestLanguageLists))
    suite.addTest(makeSuite(TestSelectorData))
    return suite

if __name__ == '__main__':
    framework()
